︠c1475f7a-adb2-4b7b-8f0a-25cbdb0cb804i︠
%md
# January 29, 2016: Explicitly compute the mod-2 representation attached to 11a

As a warm-up before returning to $\rho_{E,3}$, let's compute $\rho_{E,2}:G_{\QQ} \to \text{GL}_2(\FF_2)$ for $E=11a$.

This is more straightforward to do without getting stuck with really difficult computations that force us to learn things
more deeply.
︡af9f794b-648a-427f-9ef2-93a2fd149fc5︡︡{"done":true,"md":"# January 29, 2016: Explicitly compute the mod-2 representation attached to 11a\n\nAs a warm-up before returning to $\\rho_{E,3}$, let's compute $\\rho_{E,2}:G_{\\QQ} \\to \\text{GL}_2(\\FF_2)$ for $E=11a$.\n\nThis is more straightforward to do without getting stuck with really difficult computations that force us to learn things\nmore deeply."}
︠940ee6fa-c8ee-4b81-935d-cbdb6b383c48︠
E = EllipticCurve('11a')
E.galois_representation().is_surjective(2)
︡7df3623c-d2d1-4d89-b4a9-682064aa6a96︡︡{"stdout":"True","done":false}︡{"stdout":"\n","done":false}︡{"done":true}
︠55403f52-b74a-40e7-9c37-28afd09cb2c9︠
E.galois_representation().non_surjective()
︡a58289c5-74ed-4508-9aff-f24d3751a500︡︡{"stdout":"[5]\n","done":false}︡{"done":true}
︠15b1bb47-7c05-42a1-b9a9-2d986388023f︠
groups.matrix.GL(2,GF(2)).cardinality()
︡4d8ad792-c58f-4698-b5ab-37356282ac62︡︡{"stdout":"6\n","done":false}︡{"done":true}
︠87565c0a-801d-4979-85be-3f1b62be26b7︠
# Compute random points over QQbar...
%time
f = E.division_polynomial(2)
Ebar = E.change_ring(QQbar)
v = f.roots(ring=QQbar, multiplicities=False)
P = Ebar.lift_x(v[0])
Q = Ebar.lift_x(v[1])
P, Q
︡4d61beba-b566-48ca-a1c7-66c62c6a7a83︡︡{"stdout":"((4.346308158205395? : -1/2 : 1), (-1.673154079102698? - 1.320848922269076?*I : -1/2 : 1))\n","done":false}︡{"stdout":"CPU time: 0.06 s, Wall time: 0.06 s\n","done":false}︡{"done":true}
︠93b54495-a998-474e-ae87-dd7f66c5330e︠
# ... and try some random linear combinations and get something of degree 6.
(P[0] + P[1] - Q[0] + Q[1]).minpoly()
︡17df9bea-94e9-4e17-a8e2-8675b285269e︡︡{"stdout":"x^6 + 6*x^5 - 47*x^4 - 228*x^3 + 604*x^2 + 1680*x + 175451/16\n","done":false}︡{"done":true}
︠80850409-149e-4c86-8e24-3ee134df71d4︠
# Or just take the Galois closure of field defined by f (which luckily has degree 6).
f = E.division_polynomial(2)
K.<a> = NumberField(f,'b').galois_closure()
K
︡c9518299-a33a-4ceb-821f-65cfbe0e3b38︡︡{"stdout":"Number Field in a with defining polynomial x^6 - 68*x^4 - 120*x^3 + 1552*x^2 + 8568*x + 16316\n","done":false}︡{"done":true}
︠941b6de6-23f3-4098-81bb-cb4928bc7788︠
R = K.maximal_order(); R
︡f29ba018-03a3-4ede-9a87-315ebfd12f36︡︡{"stdout":"Maximal Order in Number Field in a with defining polynomial x^6 - 68*x^4 - 120*x^3 + 1552*x^2 + 8568*x + 16316\n","done":false}︡{"done":true}
︠bd1cc776-507b-4f6d-b6d5-1fa0b7f76790︠
EK = E.change_ring(K)
T = EK.torsion_subgroup()
T
︡143d6547-0944-42b9-a883-6c865c2d7e7c︡︡{"stdout":"Torsion Subgroup isomorphic to Z/10 + Z/2 associated to the Elliptic Curve defined by y^2 + y = x^3 + (-1)*x^2 + (-10)*x + (-20) over Number Field in a with defining polynomial x^6 - 68*x^4 - 120*x^3 + 1552*x^2 + 8568*x + 16316\n","done":false}︡{"done":true}
︠af7c3dfc-6e8a-4a49-8634-877ba358018e︠
# Basis
P0 = (T.0*5).element()
P1 = T.1.element()
P0.order(); P1.order()
P0, P1
︡cc94bca4-6aa0-44d5-90d9-11c054ccc8f3︡︡{"stdout":"2","done":false}︡{"stdout":"\n2\n","done":false}︡{"stdout":"((105/67573*a^5 - 2085/270292*a^4 - 5046/67573*a^3 + 12011/67573*a^2 + 157524/67573*a + 352835/67573 : -1/2 : 1), (723/540584*a^5 - 2931/540584*a^4 - 21585/270292*a^3 + 54779/270292*a^2 + 381035/270292*a + 504485/135146 : -1/2 : 1))\n","done":false}︡{"done":true}
︠08bed219-e1de-406e-9e19-4bcecc4ed27c︠
def frob(X):
    p = X[0].parent().characteristic()
    return X.parent()([X[0]^p, X[1]^p])

def in_terms_of_basis(X, P0mod, P1mod):
    for i in range(2):
        for j in range(2):
            if X == i*P0mod + j*P1mod:
                return [i,j]
    raise ValueError

def rhoFrob(prime):
    """
    Compute the matrix of Frob_P, where P is a prime of the ringer of integers of K.
    The 2-torsion points P0, P1 in E(K) above must be defined.

    INPUT:
       - P = prime ideal of R.
    """
    F = prime.residue_field()
    EF = E.change_ring(F)
    P0mod = EF(P0)
    P1mod = EF(P1)
    return matrix(GF(2), 2, [in_terms_of_basis(frob(Q), P0mod, P1mod) for Q in [P0mod, P1mod]]).transpose()

︡aff424bc-4cfe-411b-b871-5ec07897ea06︡︡{"done":true}
︠73c2d386-5da0-4a81-9912-221cc42fc75e︠
K.discriminant().factor()
︡086da05f-9488-4f54-8522-7a48c87b6391︡︡{"stdout":"-1 * 2^4 * 11^3\n","done":false}︡{"done":true}
︠4e6dea80-6b1a-4ef3-99db-f1df24fc7a1a︠
K.factor(3)
︡cb6a7115-fc1a-4af5-ae94-f95042624d28︡︡{"stdout":"(Fractional ideal (3, -9/5324*a^5 + 51/5324*a^4 + 323/5324*a^3 - 597/2662*a^2 - 3601/2662*a - 9277/2662)) * (Fractional ideal (3, -9/5324*a^5 + 51/5324*a^4 + 323/5324*a^3 - 597/2662*a^2 - 3601/2662*a - 11939/2662))\n","done":false}︡{"done":true}
︠33e08d6a-ad41-4835-8483-83a1b3a39187︠
def frob_report(p):
    R.<X> = GF(2)[]
    for P, e in K.factor(p):
        A = rhoFrob(P)
        print '-'*30
        print "p = %s"%p
        print A
        print A.charpoly()
        print X^2 - E.ap(p)*X + p
︡28a828e9-6146-4163-b0cd-270e2732c08e︡︡{"done":true}
︠d6108326-9bf1-4f5c-8c0a-16f64e96cc23︠
for p in primes(10):
    if 2*11%p == 0: continue
    frob_report(p)
︡5e06a825-b681-41e1-82f2-94bef244dff1︡︡{"stdout":"------------------------------\np = 3\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 3\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 5\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 5\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 7\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 7\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 7\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n","done":false}︡{"done":true}
︠945ec1e2-b8d8-458d-834c-928078b258e9︠
%time
frob_report(next_prime(10^40))
︡5fe24bd2-8c17-4144-8c58-16e20ec24256︡︡{"stdout":"------------------------------","done":false}︡{"stdout":"\np = 10000000000000000000000000000000000000121\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1","done":false}︡{"stdout":"\n------------------------------","done":false}︡{"stdout":"\np = 10000000000000000000000000000000000000121\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1","done":false}︡{"stdout":"\n","done":false}︡{"stdout":"CPU time: 0.81 s, Wall time: 0.83 s\n","done":false}︡{"done":true}
︠8631aaf5-8b47-47ba-8e23-f7adc05a9ac3︠
for p in primes(100):
    if 2*11%p == 0: continue
    frob_report(p)
︡918f1ce9-cd7c-422a-afdf-1d567d845009︡︡{"stdout":"------------------------------\np = 3\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 3\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 5\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 5\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 7\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 7\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 7\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 13\n[1 0]\n[1 1]\nx^2 + 1","done":false}︡{"stdout":"\nX^2 + 1\n------------------------------\np = 13\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 13\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 17\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 17\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 17\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 19\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 19\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 19\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 23\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 23\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 29\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 29\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 29\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 31\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 31\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 37\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 37\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 41\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 41\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 41\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 43\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 43\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 43\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 47\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1","done":false}︡{"stdout":"\nX^2 + 1\n------------------------------\np = 53\n[1 0]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 59\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 59\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 61\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 61\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 61\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 67\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 67\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 71\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 71\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 73\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 73\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 73\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 79\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 79\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 79\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 83\n[1 0]\n[1 1]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 83\n[1 1]\n[0 1]\nx^2 + 1\nX^2 + 1\n------------------------------","done":false}︡{"stdout":"\np = 83\n[0 1]\n[1 0]\nx^2 + 1\nX^2 + 1\n------------------------------\np = 89\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 89\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------\np = 97\n[0 1]\n[1 1]\nx^2 + x + 1\nX^2 + X + 1\n------------------------------","done":false}︡{"stdout":"\np = 97\n[1 1]\n[1 0]\nx^2 + x + 1\nX^2 + X + 1\n","done":false}︡{"done":true}
︠4b191e12-0384-41b4-9d7d-5f7edf963a86i︠
%md

**Remark:** Suppose $\ell$ is very small, e.g., $\ell=2$ or $\ell=3$ so we can explicitly compute $\rho_{E,\ell}$ (I hope). Here's a reflection of "something deep".  Suppose $p$ is a HUGE PRIME.  Consider the quantities:

A: $p + 1 - \#E(\FF_p) \mod \ell \in \FF_{\ell}$

B: $\text{Tr}(\rho_{E,\ell}(\text{Frob}_p)) \in \FF_\ell$.

These two quantities are equal (by the theorem I mentioned -- it is more generally called the "Eichler-Shimura relation".)  In particular,
$\#E(\FF_p) \equiv p + 1 - \text{Tr}(\rho_{E,\ell}(\text{Frob}_p)) \pmod{\ell}$.

The first is hard to compute, since for $p$ HUGE, computing $\#E(\FF_p)$ is VERY difficult.  Even computing anything at all in $E(\FF_p)$ is at least pretty hard.

On the other hand, to compute $\rho_{E,\ell}(\text{Frob}_p)$, after doing some big initial computation, you just need to find a small prime $q$ such that $\text{Frob}_p = \text{Frob}_q$, and you're done.  Finding $q$ is purely a computation in the number field $K=\QQ(E[\ell])$.  It's a lot like quadratic reciprocity, except the "congruence" there is replaced by "defining the same element of $\text{Gal}(K/\QQ)$".

**Schoof:** Incidentally, a key idea used in Schoof's groundbreaking algorithm for computing $E(\text{Frob}_p)$ for large $p$, which is at the foundation of elliptic curve cryptograph, is the relation above, which reduces computing $\#E(\FF_p)$ to the problem of computing $\text{Tr}(\rho_{E,\ell}(\text{Frob}_p)) \pmod{\ell}$ for sufficiently many primes $\ell$.
︡7b996693-e857-43d2-9c3a-958d35d2c2fc︡︡{"done":true,"md":"\n**Remark:** Suppose $\\ell$ is very small, e.g., $\\ell=2$ or $\\ell=3$ so we can explicitly compute $\\rho_{E,\\ell}$ (I hope). Here's a reflection of \"something deep\".  Suppose $p$ is a HUGE PRIME.  Consider the quantities:\n\nA: $p + 1 - \\#E(\\FF_p) \\mod \\ell \\in \\FF_{\\ell}$\n\nB: $\\text{Tr}(\\rho_{E,\\ell}(\\text{Frob}_p)) \\in \\FF_\\ell$.\n\nThese two quantities are equal (by the theorem I mentioned -- it is more generally called the \"Eichler-Shimura relation\".)  In particular,\n$\\#E(\\FF_p) \\equiv p + 1 - \\text{Tr}(\\rho_{E,\\ell}(\\text{Frob}_p)) \\pmod{\\ell}$.\n\nThe first is hard to compute, since for $p$ HUGE, computing $\\#E(\\FF_p)$ is VERY difficult.  Even computing anything at all in $E(\\FF_p)$ is at least pretty hard.\n\nOn the other hand, to compute $\\rho_{E,\\ell}(\\text{Frob}_p)$, after doing some big initial computation, you just need to find a small prime $q$ such that $\\text{Frob}_p = \\text{Frob}_q$, and you're done.  Finding $q$ is purely a computation in the number field $K=\\QQ(E[\\ell])$.  It's a lot like quadratic reciprocity, except the \"congruence\" there is replaced by \"defining the same element of $\\text{Gal}(K/\\QQ)$\".\n\n**Schoof:** Incidentally, a key idea used in Schoof's groundbreaking algorithm for computing $E(\\text{Frob}_p)$ for large $p$, which is at the foundation of elliptic curve cryptograph, is the relation above, which reduces computing $\\#E(\\FF_p)$ to the problem of computing $\\text{Tr}(\\rho_{E,\\ell}(\\text{Frob}_p)) \\pmod{\\ell}$ for sufficiently many primes $\\ell$."}
︠535b4ccf-2152-4a37-a445-83af54002e17︠
︠3887a504-6b1c-4044-a977-125de2312366︠









